var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Buffer } from "buffer";
import git from "isomorphic-git";
import { isVacantAndWriteable, pointsToLFS } from "./util";
import downloadBlobFromPointer from "./download";
import { readPointer } from "./pointers";
const SYMLINK_MODE = 40960;
/**
 * Populates LFS cache for each repository object that is an LFS pointer.
 *
 * Does not touch working directory.
 *
 * NOTE: If LFS cache path, as extracted from the pointer,
 * is not writeable at the time of download start,
 * the object will be silently skipped.
 *
 * NOTE: This function skips objects silently in case of errors.
 *
 * NOTE: onProgress currently doesn’t report loaded/total values accurately.
 */
export default function populateCache(fs, http, workDir, remoteURL, ref = "HEAD", onProgress) {
    return __awaiter(this, void 0, void 0, function* () {
        yield git.walk({
            fs,
            dir: workDir,
            trees: [git.TREE({ ref })],
            map: function lfsDownloadingWalker(filepath, entries) {
                return __awaiter(this, void 0, void 0, function* () {
                    if (entries === null || entries[0] === null) {
                        return null;
                    }
                    onProgress === null || onProgress === void 0 ? void 0 : onProgress({ phase: `skimming: ${filepath}`, loaded: 5, total: 10 });
                    const [entry] = entries;
                    const entryType = yield entry.type();
                    if (entryType === "tree") {
                        // Walk children
                        return true;
                    }
                    else if (entryType === "blob" &&
                        (yield entry.mode()) !== SYMLINK_MODE) {
                        const content = yield entry.content();
                        if (content) {
                            const buff = Buffer.from(content.buffer);
                            if (pointsToLFS(buff)) {
                                const pointer = readPointer({ dir: workDir, content: buff });
                                // Don’t even start the download if LFS cache path is not accessible,
                                // or if it already exists
                                if ((yield isVacantAndWriteable(pointer.objectPath)) === false)
                                    return;
                                onProgress === null || onProgress === void 0 ? void 0 : onProgress({
                                    phase: `downloading: ${filepath}`,
                                    loaded: 5,
                                    total: 10,
                                });
                                yield downloadBlobFromPointer(fs, remoteURL, {}, pointer);
                            }
                        }
                    }
                    return;
                });
            },
        });
    });
}
//# sourceMappingURL=populateCache.js.map